import os
import shutil
import uuid
from django.utils.unittest import TestCase
from archer.projects.models import Project, FileSystem
from archer.core import exceptions


class ProjectTestCase(TestCase):
    def setUp(self):
        uid = str(uuid.uuid4())[0:8]
        self.mount_point = ('/tmp/cvmfs-%s' % uid)
        self.fs = FileSystem.objects.create(mount_point=self.mount_point)
        os.makedirs(self.fs.mount_point)

        # def remove_mount_point():
        #     os.chmod(self.fs.mount_point, 02700)
        #     shutil.rmtree(self.fs.mount_point)
        # self.addCleanup(remove_mount_point())

        self.project1 = Project(file_system=self.fs, directory='project1')

    def test_full_path(self):
        project2 = Project(file_system=self.fs, directory='project2')
        self.assertEqual(project2.full_path(), os.path.join(self.fs.mount_point, 'project2'))

    def test_clear_dir(self):
        directory = os.path.join(self.fs.mount_point, 'project2')
        if not os.path.exists(directory):
            os.makedirs(directory)

        project2 = Project(file_system=self.fs, directory='project2')

        try:
            project2.clear_dir()
        except (IOError, OSError, exceptions.ValidationError) as e:
            print(e)
            self.fail(e)

    def test_clear_dir_no_dir(self):
        directory = os.path.join(self.fs.mount_point, 'project2')
        if os.path.exists(directory):
            shutil.rmtree(directory)

        with self.assertRaises(exceptions.ValidationError):
            p = Project(file_system=self.fs, directory='project2')
            p.clear_dir()

    def test_clear_dir_no_chmod(self):
        directory = os.path.join(self.fs.mount_point, 'project2')
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.mkdir(os.path.join(directory, 'something'))
        os.chmod(directory, 02400)

        with self.assertRaises(OSError):
            p = Project(file_system=self.fs, directory='project2')
            p.clear_dir()

    def test_clear_dir_no_mount_point_chmod(self):
        directory = os.path.join(self.fs.mount_point, 'project2')
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.mkdir(os.path.join(directory, 'something'))
        os.chmod(self.fs.mount_point, 02444)

        with self.assertRaises(exceptions.ValidationError):
            p = Project(file_system=self.fs, directory='project2')
            p.clear_dir()

    def test_subdir_none_or_empty(self):
        path = '%s/%s' % (self.mount_point, self.project1.directory)

        dir0 = self.project1.subdir()
        dir1 = self.project1.subdir(None)
        dir2 = self.project1.subdir('')
        dir3 = self.project1.subdir('.')
        dir4 = self.project1.subdir('././.')

        self.assertEqual(dir0, path)
        self.assertEqual(dir1, path)
        self.assertEqual(dir2, path)
        self.assertEqual(dir3, path)
        self.assertEqual(dir4, path)

    def test_subdir_allowed_parent(self):
        subdir = 'foo/bar'
        path = '%s/%s/%s' % (self.mount_point, self.project1.directory, subdir)

        dir = self.project1.subdir('foo/bar/../bar')

        self.assertEqual(dir, path)

    def test_subdir_disallowed_parent(self):
        with self.assertRaises(exceptions.ValidationError):
            self.project1.subdir('foo/../../foo/bar')

    def test_subdir_root_dir(self):
        with self.assertRaises(exceptions.ValidationError):
            self.project1.subdir('/tmp')

    def tearDown(self):
        self.fs.delete()
        os.chmod(self.mount_point, 00777)
        for r, d, f in os.walk(self.mount_point):
            os.chmod(r, 00777)
        shutil.rmtree(self.mount_point, ignore_errors=False)



from django.contrib.auth.middleware import RemoteUserMiddleware


class CustomHeaderMiddleware(RemoteUserMiddleware):
    """
    Authentication is done by httpd server and certificate DN is passed as a username to web application.
    """
    header = 'SSL_CLIENT_S_DN'



# Create your views here.
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.http.response import HttpResponseBadRequest
from django.shortcuts import render
from guardian.decorators import permission_required_or_403
from archer.projects.models import Project, FileSystem
from archer.custom_auth.models import User
from forms import GrantAdminForm, FileSystemForm


def index(request):
    show_admin = not User.superuser_exist()
    show_projects = Project.can_setup(request.user)
    show_filesystems = FileSystem.can_setup(request.user)
    return render(request, 'appsetup/index.html', {'show_setup_projects': show_projects,
                                                'show_setup_admin': show_admin,
                                                'show_setup_filesystems': show_filesystems,
                                                })


@login_required
def admin(request):
    if User.superuser_exist():
        messages.add_message(request, messages.ERROR, 'Cannot initialize the application. System admin already exists.')
        return HttpResponseRedirect(reverse('projects:index'))
    form = GrantAdminForm(request.POST)
    if request.method == 'GET':
        users = User.objects.all()
        return render(request, 'appsetup/admin.html', {'users': users})
    elif request.method == 'POST':
        if form.is_valid():
            try:
                user = request.user
                user.is_staff = True
                user.is_superuser = True
                user.save()
                messages.add_message(request, messages.SUCCESS, 'Admin privileges granted!')
                return HttpResponseRedirect(reverse('appsetup:index'))
            except (ValidationError, ) as e:
                messages.add_message(request, messages.ERROR, 'Failed to save user: %s' % e)

        return render(request, 'appsetup/admin.html', {'form': form, })
    else:
        return HttpResponseBadRequest()


@permission_required_or_403('projects.setup_filesystem')
def filesystems(request):
    if request.method == 'GET':
        form = FileSystemForm()
        return render(request, 'appsetup/filesystems.html', {'form': form})
    elif request.method == 'POST':
        form = FileSystemForm(request.POST)
        if form.is_valid():
            return render(request, 'appsetup/filesystems.html', {'form': form})
        else:
            return render(request, 'appsetup/filesystems.html', {'form': form})
    else:
        return HttpResponseBadRequest()


@permission_required_or_403('projects.setup_project')
def projects(request):
    return render(request, 'appsetup/projects.html')



from django.contrib import admin
from archer.custom_auth.models import User
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserChangeForm, UserCreationForm
from django.utils.translation import ugettext_lazy as _
from django import forms
from guardian.admin import GuardedModelAdmin


class CustomUserChangeForm(UserChangeForm):
    username = forms.RegexField(
        label=_("Username"), max_length=200, regex=r"^[\w.@+-=/ ]+$",
        help_text=_("Required. 200 characters or fewer. Letters, digits and "
                    "@/./+/-/_/=/ // only."),
        error_messages={
            'invalid': _("This value may contain only letters, numbers and "
                         "@/./+/-/_/=/ // characters.")})


class CustomUserCreationForm(UserCreationForm):
    username = forms.RegexField(label=_("Username"), max_length=200,
                                regex=r'^[\w.@+-=/ ]+$',
                                help_text=_("Required. 200 characters or fewer. Letters, digits and "
                                            "@/./+/-/_/=/ // only."),
                                error_messages={
                                    'invalid': _("This value may contain only letters, numbers and "
                                                 "@/./+/-/_/=/ // characters.")})

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            User._default_manager.get(username=username)
        except User.DoesNotExist:
            return username
        raise forms.ValidationError(self.error_messages['duplicate_username'])

    class Meta(UserCreationForm.Meta):
        model = User


class CustomUserAdmin(UserAdmin, GuardedModelAdmin):
    form = CustomUserChangeForm
    add_form = CustomUserCreationForm


admin.site.register(User, CustomUserAdmin)



import os
import configurations.importer

configurations.importer.install()

from common import Common
from dev import Dev
from production import Production
from test import Test

if os.environ['DJANGO_CONFIGURATION'] == 'Production':
    Production.load_cfg()
# hack is needed to make sniffer/nosetests command work with django-configurations plugin
elif os.environ['DJANGO_CONFIGURATION'] == 'Test':
    ANONYMOUS_USER_ID = Test.ANONYMOUS_USER_ID
    SOUTH_TESTS_MIGRATE = Test.SOUTH_TESTS_MIGRATE
    INSTALLED_APPS = Test.INSTALLED_APPS
    TEST_RUNNER = Test.TEST_RUNNER
    SECRET_KEY = Test.SECRET_KEY
    DATABASES = Test.DATABASES
    TEST_RUNNER = Test.TEST_RUNNER
    PASSWORD_HASHERS = Test.PASSWORD_HASHERS
    DEFAULT_FILE_STORAGE = Test.DEFAULT_FILE_STORAGE


from django.core import validators
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, UserManager, SiteProfileNotAvailable
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.http import urlquote
import re
import warnings
from django.utils.translation import ugettext_lazy as _


class AbstractUser(AbstractBaseUser, PermissionsMixin):
    """
    This class is just a copy of default Django AbstractUser with couple changes to validation and field lengths.
    Copying all AbstractUser and User seemed to be better approach than any monkey patching.

    Username is a certificate DN so it has to allow characters like '/'.
    Additionally it is quite long so the length of username was increased to 200
    and might be change in the future if needed.


    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.

    Username, password and email are required. Other fields are optional.
    """
    username = models.CharField(_('username'), max_length=200, unique=True,
                                help_text=_('Required. 200 characters or fewer. Letters, numbers and '
                                            '@/./+/-/_//=/ characters'),
                                validators=[
                                    validators.RegexValidator(re.compile('^[\w.@+-=/ ]+$'), _('Enter a valid username.'), 'invalid')
                                ])
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('email address'), blank=True)
    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin '
                                               'site.'))
    is_active = models.BooleanField(_('active'), default=True,
                                    help_text=_('Designates whether this user should be treated as '
                                                'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = UserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_absolute_url(self):
        return "/users/%s/" % urlquote(self.username)

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    def get_profile(self):
        """
        Returns site-specific profile for this user. Raises
        SiteProfileNotAvailable if this site does not allow profiles.
        """
        warnings.warn("The use of AUTH_PROFILE_MODULE to define user profiles has been deprecated.",
                      PendingDeprecationWarning)
        if not hasattr(self, '_profile_cache'):
            from django.conf import settings

            if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
                raise SiteProfileNotAvailable(
                    'You need to set AUTH_PROFILE_MODULE in your project '
                    'settings')
            try:
                app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
            except ValueError:
                raise SiteProfileNotAvailable(
                    'app_label and model_name should be separated by a dot in '
                    'the AUTH_PROFILE_MODULE setting')
            try:
                model = models.get_model(app_label, model_name)
                if model is None:
                    raise SiteProfileNotAvailable(
                        'Unable to load the profile model, check '
                        'AUTH_PROFILE_MODULE in your project settings')
                self._profile_cache = model._default_manager.using(
                    self._state.db).get(user__id__exact=self.id)
                self._profile_cache.user = self
            except (ImportError, ImproperlyConfigured):
                raise SiteProfileNotAvailable
        return self._profile_cache


class User(AbstractUser):
    """
    This class is just a copy of default Django User with couple changes to validation and field lengths.
    Copying all AbstractUser and User seemed to be better approach than any monkey patching.

    Users within the Django authentication system are represented by this
    model.

    Username, password and email are required. Other fields are optional.
    """
    @classmethod
    def superuser_exist(cls):
        return User.objects.filter(is_superuser=True, is_staff=True).count() > 0

    class Meta:
        app_label = 'custom_auth'
        swappable = 'AUTH_USER_MODEL'



from django.conf.urls import patterns, include, url

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
from archer.projects import views

admin.autodiscover()

urlpatterns = patterns('',
                       # Examples:
                       # url(r'^$', 'archer.views.home', name='home'),

                       # Uncomment the admin/doc line below to enable admin documentation:
                       # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

                       # url(r'^grappelli/', include('grappelli.urls')),
                       url(r'^$', include('archer.projects.urls'), name='index'),
                       url(r'^setup/', include('archer.appsetup.urls', namespace='appsetup')),
                       url(r'^projects/', include('archer.projects.urls', namespace='projects')),
                       url(r'^packages/', include('archer.packages.urls', namespace='packages')),
                       # Uncomment the next line to enable the admin:
                       url(r'^admin/', include(admin.site.urls)),
                       url(r'^unauthenticated$', views.unauthenticated, name='unauthenticated')
)



# noinspection PyUnresolvedReferences
import re
from archer.settings.common import Common


class Production(Common):
    DEBUG = True
    TEMPLATE_DEBUG = DEBUG

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
            'NAME': '%s/db/uploader.sqlite3' % Common.PROJECT_ROOT,  # Or path to database file if using sqlite3.
            'USER': '',  # Not used with sqlite3.
            'PASSWORD': '',  # Not used with sqlite3.
            'HOST': '',  # Set to empty string for localhost. Not used with sqlite3.
            'PORT': '',  # Set to empty string for default. Not used with sqlite3.
        }
    }
    @classmethod
    def load_cfg(cls):
        import ConfigParser
        import os

        def cast(value):
            if value.isdigit():
                return int(value)
            elif value.lower() in ('true', 'false'):
                return value.lower() == 'true'
            else:
                return value

        def set(option, value):
            setattr(cls, option.upper(), value)

        OPTIONS_AVAILABLE = {
            'path': ['PROJECT_ROOT', 'MEDIA_ROOT', 'STATIC_ROOT'],
            'url': ['HOSTNAME', 'MEDIA_URL', 'STATIC_URL'],
            'security': ['SECRET_KEY', 'CSRF_MIDDLEWARE_SECRET', 'ALLOWED_HOSTS'],
            'debug': ['DEBUG', 'TEMPLATE_DEBUG', 'VIEW_TEST', 'INTERNAL_IPS', 'SKIP_CSRF_MIDDLEWARE'],
            'misc': None,
        }
        LIST_TYPES = (
            ('debug', 'INTERNAL_IPS',),
            ('security', 'ALLOWED_HOSTS',),
        )

        config = ConfigParser.SafeConfigParser()
        config.read(['/etc/uploader.cfg', os.path.expanduser('~/.uploader.cfg'), './archer/settings/production.cfg'])
        for section in config.sections():
            if section == 'database':
                default_db = {}
                for option in config.options(section):
                    default_db[option.upper()] = config.get(section, option)
                set('DATABASE', {'default': default_db})
            elif section == 'logging':
                print 'TBD'
            else:
                if section not in OPTIONS_AVAILABLE.keys():
                    raise ValueError('Unrecognized section: [%s]. Perhaps, [misc] should be used instead.' % section)

                for option in config.options(section):
                    option = option.upper()
                    if OPTIONS_AVAILABLE[section] is not None and option not in OPTIONS_AVAILABLE[section]:
                        raise ValueError('Option "%s" is not available for section [%s]' % (option, section))
                    value = config.get(section, option)

                    if (section, option,) in LIST_TYPES:
                        set(option, tuple(re.split(',\s*', value)))
                    else:
                        set(option, cast(value))



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import render, get_object_or_404
from guardian.decorators import permission_required_or_403
from django.core.exceptions import PermissionDenied

from archer.projects.models import Project
from archer.packages.models import Package


def show(request, package_id):
    package = get_object_or_404(Package, id=package_id)

    project = Project.objects.get(pk=package.project_id)
    if not request.user.has_perm('projects.view_project', project):
        raise PermissionDenied

    can_deploy = request.user.has_perm('packages.deploy_package', package) \
        and request.user.has_perm('projects.deploy_package', project)
    can_remove = request.user.has_perm('packages.remove_package', package)
    return render(request, 'packages/show.html', {
        'package': package,
        'files': package.get_file_list(),
        'can_deploy': can_deploy,
        'can_remove': can_remove})


@permission_required_or_403('packages.deploy_package', (Package, 'pk', 'package_id'))
def deploy(request, package_id):
    package = get_object_or_404(Package, id=package_id)

    project = Project.objects.get(pk=package.project_id)
    if not request.user.has_perm('projects.deploy_package', project):
        raise PermissionDenied
    if not package.can_deploy():
        messages.add_message(request, messages.ERROR, 'Cannot deploy a package!')
        return render(request, 'packages/show.html', {'package': package})
    else:
        # for p in Package.objects.filter(status=Package.Status.deployed):
        #     p.undeploy()
        try:
            if request.POST.has_key('subdir'):
                result = package.deploy(request.POST['subdir'])
            else:
                result = package.deploy()
            if result:
                messages.add_message(request, messages.INFO, 'Package deployed!')
            else:
                messages.add_message(request, messages.ERROR, 'Error during deployment of the package.')
        except IOError as e:
            messages.add_message(request, messages.ERROR, 'Error during deployment of the package: ' + str(e))

        return render(request, 'packages/show.html', {'package': package})


@permission_required_or_403('packages.remove_package', (Package, 'pk', 'package_id'))
def remove(request, package_id):
    package = get_object_or_404(Package, id=package_id)

    if package.can_remove():
        if package.status == Package.Status.deployed:
            Package.clear_dir(package.project.full_path())
        try:
            if package.remove():
                messages.add_message(request, messages.INFO, 'Package file successfully deleted!')
            else:
                messages.add_message(request, messages.WARNING,
                                     'Package file was already deleted from the file system. Mark as deleted.')
        except IOError as e:
            messages.add_message(request, messages.ERROR, e)
    else:
        messages.add_message(request, messages.ERROR, 'Cannot delete package file!')
    return render(request, 'packages/show.html', {'package': package})



#!/home/vwa13376/.virtualenvs/uploader3/bin/python
import os
import sys


def main():
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'archer.settings')

    if len(sys.argv) > 1 and sys.argv[1] == 'test':
        os.environ.setdefault('DJANGO_CONFIGURATION', 'Test')
        os.environ.setdefault('REUSE_DB', '1')
    else:
        os.environ.setdefault('DJANGO_CONFIGURATION', 'Dev')

    print 'DJANGO_SETTINGS_MODULE = %s' % os.environ['DJANGO_SETTINGS_MODULE']
    print 'DJANGO_CONFIGURATION = %s' % os.environ['DJANGO_CONFIGURATION']

    import configurations.importer
    configurations.importer.install()

    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()



class ApplicationError(Exception):
    """
    Raised by application logic.l
    """
    pass


class ArgumentError(ApplicationError):
    """
    Raised on unexpected actions which should not occur during normal usage,
    e.g. an user sends crafted HTTP header or opens URL which link does not exist anywhere in the application.
    """
    pass


class ValidationError(ApplicationError):
    """
    Raised when data provided by user does not match the requirements,
    e.g. an user sends /root or parent directory as the argument of an action.
    """
    pass



import os
import re
import shutil
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
from django.db import models
from django.db.models import Q
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
from guardian.shortcuts import assign_perm
from archer.core import exceptions


def validator_dir_exists(path):
    if not os.path.exists(path):
        raise ValidationError('%s does not exist!' % path, code='dir_not_exist')
    if not os.path.isdir(path):
        raise ValidationError('%s is not a directory!' % path, code='dir_not_dir')
    if not os.access(path, os.W_OK | os.X_OK):
        raise ValidationError('%s is not writable!' % path, code='dir_not_writable')


def validator_relative_path(path):
    rel_path = os.path.relpath(path)
    if re.match('^(\.\./.+|/.+|\.\.)$', rel_path):
        raise ValidationError(_('Directory path name must be relative path beneath the file system.'), code='dir_name')


class FileSystem(models.Model):
    alias = models.CharField(max_length=2000, blank=True)
    mount_point = models.CharField(max_length=2000, null=False, blank=False, unique=True,
                                   validators=[validator_dir_exists])

    class Meta:
        permissions = (
            ('setup_filesystem', 'Setup file system'),
        )

    def validate_unique(self, *args, **kwargs):
        """
            Validates if "alias" is unique within scope of "mount_point".
            "alias" mustn't be equal to any other "alias" or "mount_point".
            Same applies to "mount_point" which mustn't equal to any other "alias" except for the same entity.
            """

        def validate_uniqueness_of_alias_and_mount_point(rethrow_error=None):
            uq_mount = self.__class__.objects.filter(
                Q(alias=self.mount_point)
            )
            uq_alias = self.__class__.objects.filter(
                Q(mount_point=self.alias)
            )

            errors = {}
            if not self._state.adding and self.pk is not None:
                uq_mount = uq_mount.exclude(pk=self.pk)
                uq_alias = uq_alias.exclude(pk=self.pk)

            if uq_mount.exists():
                errors['mount_point'] = (
                    'Field is the same as "Alias" for other file system (alias="%s", mount_point="%s")!' % (
                        uq_mount.get().alias, uq_mount.get().mount_point), )
            if uq_alias.exists():
                errors['alias'] = (
                    'Field is the same as "Mount point" for other file system (alias="%s", mount_point="%s")!' % (
                        uq_alias.get().alias, uq_alias.get().mount_point), )
            if len(errors) > 0:
                errors[NON_FIELD_ERRORS] = ('To avoid confusion choose different name!',)
                if rethrow_error is None:
                    rethrow_error = ValidationError(errors)
                else:
                    rethrow_error.message_dict.update(errors)
            if rethrow_error is not None:
                return rethrow_error

        try:
            super(FileSystem, self).validate_unique(*args, **kwargs)
            e = validate_uniqueness_of_alias_and_mount_point()
            if e is not None:
                raise e
        except ValidationError as e:
            raise validate_uniqueness_of_alias_and_mount_point(e)

    def alias_path(self):
        if self.alias is None or len(self.alias) == 0:
            return self.mount_point
        else:
            return self.alias

    @classmethod
    def can_setup(cls, user):
        return user is not None and user.is_superuser and user.is_staff

    def __unicode__(self):
        return self.alias_path()


class Project(models.Model):
    file_system = models.ForeignKey(FileSystem, null=False)
    directory = models.CharField(max_length=200, null=False, blank=False, validators=[validator_relative_path])

    class Meta:
        unique_together = ('file_system', 'directory')
        permissions = (
            ('view_project', 'View project'),
            ('upload_package', 'Upload package'),
            ('deploy_package', 'Deploy package'),
            ('make_directory', 'Make directory'),
            ('remove_directory', 'Remove directory'),
            ('remove_file', 'Remove file'),
            ('setup_project', 'Setup project'),
        )

    def full_path(self):
        return os.path.join(self.file_system.mount_point, self.directory)

    def alias_path(self):
        return os.path.join(self.file_system.alias_path(), self.directory)

    def subdir(self, subdir=''):
        """
        Returns absolute path to subdirectory of `self.directory`.
        Raises archer.core.exceptions.ValidationError if `subdir` is not a offspring of `self.directory`.
        """
        if subdir is None or len(subdir) == 0:
            return self.full_path()
        if subdir.startswith('/'):
            raise exceptions.ValidationError('Cannot provide root path "%s" as subdir!' % subdir)
        if os.path.normpath(subdir).startswith('..'):
            raise exceptions.ValidationError('Cannot privide a path "%s" pointing to parent directory!' % subdir)
        return os.path.abspath(os.path.join(self.full_path(), subdir))

    def clear_dir(self, subdir=''):
        """
        Removes all files in project's directory.
        """
        if not os.path.isdir(self.file_system.mount_point):
            raise exceptions.ValidationError("mount_point %s is not a directory" % self.file_system.mount_point)
        if not os.path.isdir(self.full_path()):
            raise exceptions.ValidationError("project %s is not a directory" % self.full_path())
        path = self.subdir(subdir)
        if not os.path.isdir(path):
            raise exceptions.ValidationError("%s is not a directory" % path)
        for root, dirs, files in os.walk(path):
            for f in files:
                os.unlink(os.path.join(root, f))
            for d in dirs:
                shutil.rmtree(os.path.join(root, d))
        return True

    @classmethod
    def can_setup(cls, user):
        return user is not None and user.is_superuser and user.is_staff and FileSystem.objects.count() > 0

    def __unicode__(self):
        return self.alias_path()


class MyModel(models.Model):
    field1 = models.TextField()
    field2 = models.IntegerField()


@receiver(post_save, sender=Project)
def project_create_directory(sender, **kwargs):
    path = kwargs['instance'].full_path()
    if not os.path.exists(path):
        os.makedirs(path, mode=0755)


@receiver(post_delete, sender=Project)
def project_delete_directory(sender, **kwargs):
    path = kwargs['instance'].full_path()
    if os.path.exists(path):
        shutil.rmtree(path, ignore_errors=True)


@receiver(post_save, sender=Project)
def project_create_project_group(sender, **kwargs):
    instance = kwargs['instance']
    group, created = Group.objects.get_or_create(name=instance.alias_path())
    for permission in ['view_project', 'upload_package', 'deploy_package', 'make_directory', 'remove_directory']:
        assign_perm('projects.%s' % permission, group, instance)


@receiver(post_delete, sender=Project)
def project_delete_project_group(sender, **kwargs):
    instance = kwargs['instance']
    try:
        group = Group.objects.get(name=instance.alias_path())
        group.delete()
    except Group.DoesNotExist:
        pass




# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Package.file'
        db.alter_column(u'packages_package', 'file', self.gf('django.db.models.fields.files.FileField')(max_length=1024))

    def backwards(self, orm):

        # Changing field 'Package.file'
        db.alter_column(u'packages_package', 'file', self.gf('django.db.models.fields.files.FileField')(max_length=100))

    models = {
        u'packages.package': {
            'Meta': {'object_name': 'Package'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '1024'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.Project']"}),
            'status': ('django.db.models.fields.IntegerField', [], {}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'projects.filesystem': {
            'Meta': {'object_name': 'FileSystem'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mount_point': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '2000'})
        },
        u'projects.project': {
            'Meta': {'unique_together': "(('file_system', 'directory'),)", 'object_name': 'Project'},
            'directory': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'file_system': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.FileSystem']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['packages']


import os
import re

from bootstrap_toolkit.widgets import BootstrapFileInput
from django import forms
from django.utils.translation import ugettext_lazy as _

from archer.packages.models import Package
from models import Project


class UploadFileForm(forms.ModelForm):
    class Meta:
        model = Package
        fields = ['file']
        widgets = {
            'file': BootstrapFileInput(format_type='simple'),
        }


class RemoveDirectoryForm(forms.Form):
    pass


class RemoveFileForm(forms.Form):
    pass


class MakeDirectoryForm(forms.Form):
    new_directory = forms.CharField(max_length=200, required=True)

    def __init__(self, parent_directory, *args, **kwargs):
        self.parent_directory = parent_directory
        super(MakeDirectoryForm, self).__init__(*args, **kwargs)

    def clean_new_directory(self):
        new_directory = self.cleaned_data['new_directory']
        if not len(new_directory) > 0:
            raise forms.ValidationError('Directory name cannot be empty!')
        if not re.match('^([\.\-]|\w)+$', new_directory):
            raise forms.ValidationError(_('Directory name must consist of ' +
                                          'alphanumeric characters, hyphens, ' +
                                          'dots or underscores only! (([\.\-]|\w)+)'), code='dir_name')

        dir_full_path = os.path.join(self.parent_directory, new_directory)
        if os.path.exists(dir_full_path):
            # raise forms.ValidationError(_('Directory "%(dir)s" already exists!'),
            #                             params={'dir': new_directory},) # Django 1.6 only
            raise forms.ValidationError(_('Directory "%s" already exists!') % new_directory,)
        return new_directory


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'FileSystem.alias'
        db.add_column(u'projects_filesystem', 'alias',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=2000, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'FileSystem.alias'
        db.delete_column(u'projects_filesystem', 'alias')


    models = {
        u'projects.filesystem': {
            'Meta': {'object_name': 'FileSystem'},
            'alias': ('django.db.models.fields.CharField', [], {'max_length': '2000', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mount_point': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '2000'})
        },
        u'projects.project': {
            'Meta': {'unique_together': "(('file_system', 'directory'),)", 'object_name': 'Project'},
            'directory': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'file_system': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.FileSystem']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['projects']


import os
from pprint import pprint, pformat
import shutil
from django.contrib import messages

from django.core.urlresolvers import reverse

from django.http.response import HttpResponseRedirect, HttpResponseBadRequest
from django.shortcuts import render, render_to_response, get_object_or_404
from django.template import loader
from django.utils.decorators import classonlymethod
from django.utils.safestring import mark_safe
from guardian.decorators import permission_required_or_403
from django.template.context import RequestContext
from django.views.generic import View
import re
import guardian.shortcuts

from archer.core.decorators import class_view_decorator
from archer.projects.forms import UploadFileForm, MakeDirectoryForm, RemoveDirectoryForm, RemoveFileForm
from archer.projects.models import Project
from archer.packages.models import Package
from archer.core import exceptions

NUMBER_OF_PACKAGES = 5


def unauthenticated(request):
    return render(request, 'unauthenticated.html')


def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=False):
    if user.is_authenticated():
        return guardian.shortcuts.get_objects_for_user(user, perms, klass, use_groups, any_perm)
    return []


def index(request):
    debug_info = {'user': pformat(request.user.__dict__),
                  'environ': pformat(dict(os.environ.items())),
                  'meta': pformat(dict(request.META))}
    projects = get_objects_for_user(request.user, 'projects.view_project')
    package_sets = [(project,
                     request.user.has_perm('projects.upload_package', project),
                     [package for package in project.package_set.order_by('-id')],
                    ) for project in projects]
    context = {'package_sets': package_sets, 'debug_info': debug_info}
    return render(request, 'projects/index.html', context)


@class_view_decorator(permission_required_or_403('projects.upload_package', (Project, 'pk', 'project_id')))
class UploadView(View):
    def get(self, request, project_id):
        form = UploadFileForm()
        return render_to_response('projects/upload.html',
                                  {'form': form, 'project_id': project_id},
                                  context_instance=RequestContext(request))

    def post(self, request, project_id):
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            from guardian.shortcuts import assign_perm
            # handle files
            pprint(form)
            package = form.save(commit=False)
            package.status = Package.Status.uploaded
            package.project_id = project_id
            package.save()
            assign_perm('packages.deploy_package', request.user, package)
            assign_perm('packages.remove_package', request.user, package)
            return HttpResponseRedirect(reverse('projects:show', args=[project_id]))
        return render_to_response('projects/upload.html',
                                  {'form': form, 'project_id': project_id},
                                  context_instance=RequestContext(request))


class ModifyDirectory(View):
    def validate_directory(self, project_id, path):
        project, full_path = self.__validate_format(project_id, path)
        if not os.path.isdir(full_path):
            raise exceptions.ArgumentError('"%s" is not a directory' % full_path)
        if not os.path.exists(full_path):
            raise exceptions.ArgumentError('Directory "%s" does not exist' % full_path)
        return project, full_path

    def validate_file(self, project_id, path):
        project, full_path = self.__validate_format(project_id, path)
        if not os.path.isfile(full_path):
            raise exceptions.ArgumentError('"%s" is not a file' % full_path)
        if not os.path.exists(full_path):
            raise exceptions.ArgumentError('File "%s" does not exist' % full_path)
        return project, full_path

    def __validate_format(self, project_id, path):
        if path.startswith('/'):
            raise exceptions.ValidationError('File or directory "%s" cannot start with "/"' % path)
        project = get_object_or_404(Project, pk=project_id)
        if re.search('(\.\./|/\.\.)', path):
            raise exceptions.ValidationError('File or directory "%s" contains ".."' % path)
        full_path = os.path.join(project.full_path(), path)
        return project, full_path

# TODO: use django forms
@permission_required_or_403('projects.deploy_package', (Project, 'pk', 'project_id'))
def deploy(request, project_id, path):
    def validate():
        if path.startswith('/'):
            raise exceptions.ValidationError('Directory "%s" cannot start with "/"' % path)
        project = get_object_or_404(Project, pk=project_id)
        if re.search('(\.\./|/\.\.)', path):
            raise exceptions.ValidationError('%s contains ".."' % path)
        project_path = project.full_path()
        parent_directory = os.path.join(project_path, path)
        # if os.path.commonprefix([parent_directory, project_path]) != project_path:
        #     raise
        if not os.path.isdir(parent_directory):
            raise exceptions.ArgumentError('"%s" is not a directory' % parent_directory)
        if not os.path.exists(parent_directory):
            raise exceptions.ArgumentError('"%s" does not exist' % parent_directory)
        return project, parent_directory

    validate()
    if request.POST:
        if 'package' not in request.POST:
            return HttpResponseBadRequest()
        if not request.POST['package'].isdigit():
            messages.add_message(request, messages.ERROR,
                                 'package with id = "%s" does not exist!' % request.POST['package'])
            return HttpResponseRedirect(reverse('projects:deploy', args=[project_id, path]))
        package_id = request.POST['package']
        package = Package.objects.get(pk=package_id)

        if package.can_deploy():
            if request.user.has_perm('packages.deploy_package', package):
                try:
                    deployed = package.deploy(path)
                    if deployed:
                        messages.add_message(request, messages.SUCCESS,
                                             'Package "%s" successfully deployed in "%s" directory' % (package, path))
                    else:
                        messages.add_message(request, messages.ERROR, 'Cannot deploy a package "%s"!' % package)
                except (IOError, exceptions.ValidationError, OSError) as e:
                    messages.add_message(request, messages.ERROR,
                                         'Error while deploying a package "%s": "%s"' % (package, e))
            else:
                messages.add_message(request, messages.ERROR,
                                     'Does not have permission to deploy package "%s"!' % package)
        else:
            messages.add_message(request, messages.ERROR, 'Cannot deploy a package "%s"!' % package)
        return HttpResponseRedirect(reverse('projects:show', args=[project_id]))

    else:
        projects_packages = Package.objects.filter(project_id=project_id)
        packages = [package for package in projects_packages if
                    request.user.has_perm('packages.deploy_package', package) and package.can_deploy()
        ]
        return render_to_response('projects/deploy.html',
                                  {'packages': packages,
                                   'project_id': project_id,
                                   'path': path,
                                  },
                                  context_instance=RequestContext(request))


class Remove(ModifyDirectory):
    def _get(self, request, project, path, to_remove,
             form_class=RemoveDirectoryForm,
             template='projects/rmdir.html',
             text='directory'):
        form = form_class(to_remove)
        try:
            return render_to_response(template,
                                      {'project_id': project.id,
                                       'form': form,
                                       'path': path,
                                      },
                                      context_instance=RequestContext(request))
        except exceptions.ApplicationError as e:
            messages.add_message(request, messages.ERROR, 'Could not delete %s: %s' % (text, e))
            return HttpResponseRedirect(reverse('projects:show', args=[project.id]))

    def _post(self, request, project, path, to_remove,
              form_class=RemoveDirectoryForm,
              template='',
              text='directory'):
        form = form_class(to_remove, request.POST)
        try:
            if os.path.isdir(to_remove):
                shutil.rmtree(to_remove)
            elif os.path.isfile(to_remove):
                os.remove(to_remove)
            else:
                raise exceptions.ArgumentError('How did that happen? Not a file nor directory!')
            messages.add_message(request, messages.SUCCESS,
                                 '%s "%s" successfully removed.' % (text.title(), to_remove))
        except (exceptions.ApplicationError, OSError) as e:
            messages.add_message(request, messages.ERROR,
                                 'Could not delete %s "%s": %s' % (text, to_remove, e))
        return HttpResponseRedirect(reverse('projects:show', args=[project.id]))


@class_view_decorator(permission_required_or_403('projects.remove_file', (Project, 'pk', 'project_id')))
class RemoveFile(Remove):
    def get(self, request, project_id, path):
        project, to_remove = self.validate_file(project_id, path)
        return super(RemoveFile, self)._get(request, project, path, to_remove,
                                            form_class=RemoveFileForm,
                                            template='projects/rm.html',
                                            text='file')

    def post(self, request, project_id, path):
        project, to_remove = self.validate_file(project_id, path)
        return super(RemoveFile, self)._post(request, project, path, to_remove,
                                             form_class=RemoveFileForm,
                                             template='projects/rm.html',
                                             text='file')


@class_view_decorator(permission_required_or_403('projects.remove_directory', (Project, 'pk', 'project_id')))
class RemoveDirectory(Remove):
    def get(self, request, project_id, path):
        project, to_remove = self.validate_directory(project_id, path)
        return super(RemoveDirectory, self)._get(request, project, path, to_remove,
                                                 form_class=RemoveDirectoryForm,
                                                 template='projects/rmdir.html',
                                                 text='directory')

    def post(self, request, project_id, path):
        project, to_remove = self.validate_directory(project_id, path)
        return super(RemoveDirectory, self)._post(request, project, path, to_remove,
                                                  form_class=RemoveDirectoryForm,
                                                  template='projects/rmdir.html',
                                                  text='directory')


@class_view_decorator(permission_required_or_403('projects.make_directory', (Project, 'pk', 'project_id')))
class MakeDirectory(ModifyDirectory):
    def get(self, request, project_id, path):
        project, parent = self.validate_directory(project_id, path)
        form = MakeDirectoryForm(parent)
        return render_to_response('projects/mkdir.html',
                                  {'project_id': project.id,
                                   'form': form,
                                   'path': path,
                                  },
                                  context_instance=RequestContext(request))

    def post(self, request, project_id, path):
        project, parent = self.validate_directory(project_id, path)
        form = MakeDirectoryForm(parent, request.POST)
        if form.is_valid():
            try:
                new_directory = request.POST['new_directory']
                dir_full_path = os.path.join(parent, new_directory)
                os.mkdir(dir_full_path, 00755)
                messages.add_message(request, messages.SUCCESS,
                                     'Directory "%s" successfully created.' % dir_full_path)
            except OSError as e:
                messages.add_message(request, messages.ERROR,
                                     'Could not create directory "%s": %s' % (dir_full_path, e))
            return HttpResponseRedirect(reverse('projects:show', args=[project_id]))
        return render_to_response('projects/mkdir.html',
                                  {'form': form,
                                   'path': path,
                                   'project_id': project.id},
                                  context_instance=RequestContext(request))


@permission_required_or_403('projects.view_project', (Project, 'pk', 'project_id'))
def show(request, project_id, path=''):
    SHOW_PRE_FILES_THRESHOLD = 100

    project = Project.objects.get(pk=project_id)
    can_upload = request.user.has_perm('projects.upload_package', project)
    can_deploy = request.user.has_perm('projects.deploy_package', project)
    path = project.full_path()

    def index_maker():
        def _index(root):
            files = os.listdir(root)
            files_only = []
            relative_path = ''
            for mfile in files:
                full_file_path = os.path.join(root, mfile)
                t = os.path.join(root, mfile)
                if os.path.isdir(t):
                    relative_path = full_file_path[len(path) + 1:]
                    yield loader.render_to_string('tree/_folder.html',
                                                  {'file': mfile + '/',
                                                   'subfiles': _index(os.path.join(root, t)),
                                                   'can_upload': can_upload,
                                                   'can_deploy': can_deploy,
                                                   'project_id': project_id,
                                                   'full_path': full_file_path,
                                                   'path': relative_path,
                                                  })
                    continue
                files_only += [mfile]
            if len(files_only) > 0:
                pre = len(files_only) > SHOW_PRE_FILES_THRESHOLD
                relative_path = root[len(path) + 1:]
                if pre:
                    files_only = "\n".join(files_only)
                yield loader.render_to_string('tree/_files.html',
                                              {'files': files_only,
                                               'project_id': project_id,
                                               'path': relative_path,
                                               'pre': pre,
                                              })
        if not os.path.isdir(path):
            return None
        return _index(path)

    return render_to_response('projects/show.html',
                              {'project': project,
                               'can_upload': can_upload,
                               'can_deploy': can_deploy,
                               'packages': [package for package in project.package_set.order_by('-id')],
                               'files': index_maker(),
                               'path': path,
                              },
                              context_instance=RequestContext(request))


from django.utils.unittest import TestCase


class PackageTestCase(TestCase):
    def setUp(self):
        pass
        # uid = str(uuid.uuid4())[0:8]
        # self.fs = FileSystem.objects.create(mount_point=('/tmp/cvmfs-%s' % uid))
        # self.project = Project(file_system=self.fs, directory='project')
        # os.makedirs(self.fs.mount_point)

    def test_statuses(self):
        pass

    def test_filepath(self):
        pass
        # file = open(os.path.join(self.project.full_path(), 'filename.tar.gz'), 'w')
        # file.close()
        #
        # p = Package(project=self.project, file=file)
        #
        # self.assertEqual(p.filepath(), os.path.join())

    def test_filename(self):
        pass

    def test_get_file_list(self):
        pass

    def test_deploy(self):
        pass

    def test_remove(self):
        pass


from django.conf.urls import patterns, url

from archer.projects import views

urlpatterns = patterns('',
                       url(r'^$', views.index, name='index'),
                       url(r'^(?P<project_id>\d+)/path=(?P<path>\S+)$', views.show, name='show'),
                       url(r'^(?P<project_id>\d+)/$', views.show, name='show'),
                       url(r'^(?P<project_id>\d+)/upload/$', views.UploadView.as_view(), name='upload'),
                       url(r'^(?P<project_id>\d+)/deploy=(?P<path>\S+)$', views.deploy, name='deploy'),

                       url(r'^(?P<project_id>\d+)/mkdir=(?P<path>\S+)$', views.MakeDirectory.as_view(), name='mkdir'),
                       url(r'^(?P<project_id>\d+)/rmdir=(?P<path>\S+)$', views.RemoveDirectory.as_view(), name='rmdir'),
                       url(r'^(?P<project_id>\d+)/rm=(?P<path>\S+)$', views.RemoveFile.as_view(), name='rm'),
                       )



from django.contrib import admin
from guardian.admin import GuardedModelAdmin
from archer.packages.models import Package


class PackageAdmin(GuardedModelAdmin):
    list_display = ('project', 'file', 'status', 'created_at', 'updated_at')

admin.site.register(Package, admin_class=PackageAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Package'
        db.create_table(u'packages_package', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['projects.Project'])),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('status', self.gf('django.db.models.fields.IntegerField')()),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'packages', ['Package'])


    def backwards(self, orm):
        # Deleting model 'Package'
        db.delete_table(u'packages_package')


    models = {
        u'packages.package': {
            'Meta': {'object_name': 'Package'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.Project']"}),
            'status': ('django.db.models.fields.IntegerField', [], {}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'projects.filesystem': {
            'Meta': {'object_name': 'FileSystem'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mount_point': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '2000'})
        },
        u'projects.project': {
            'Meta': {'unique_together': "(('file_system', 'directory'),)", 'object_name': 'Project'},
            'directory': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'file_system': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.FileSystem']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['packages']


from django.conf.urls import patterns, url

from archer.packages import views

urlpatterns = patterns('',
                       url(r'^(?P<package_id>\d+)/$', views.show, name='show'),
                       url(r'^(?P<package_id>\d+)/deploy/$', views.deploy, name='deploy'),
                       url(r'^(?P<package_id>\d+)/remove/$', views.remove, name='remove'),
)


# noinspection PyUnresolvedReferences
import os
from configurations import importer

importer.install()

from archer.settings.common import Common


class Test(Common):
    DEBUG = True
    TEMPLATE_DEBUG = DEBUG
    # in memory doesn't work for nosetests
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
            'TEST_NAME': ':memory:',
        },
    }

    # syncdb should be faster than using South migrations
    SOUTH_TESTS_MIGRATE = False

    INSTALLED_APPS = Common.INSTALLED_APPS + (
        'django_nose',
    )

    TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

    # Use weak hashes to increase tests speed
    PASSWORD_HASHERS = (
        'django.contrib.auth.hashers.MD5PasswordHasher',
    )

    DEFAULT_FILE_STORAGE = 'inmemorystorage.InMemoryStorage'



from django import forms
from archer.projects.models import FileSystem


class GrantAdminForm(forms.Form):
    pass


class FileSystemForm(forms.ModelForm):
    class Meta:
        model = FileSystem
        fields = ['alias', 'mount_point']



from django import forms
from django.contrib import admin
from guardian.admin import GuardedModelAdmin
from archer.projects.models import FileSystem, Project


class FileSystemAdminForm(forms.ModelForm):
    class Meta:
        model = FileSystem


class ProjectAdmin(GuardedModelAdmin):
    list_display = ('__unicode__', 'file_system', 'directory')


class FileSystemAdmin(GuardedModelAdmin):
    list_display = ('__unicode__', 'alias', 'mount_point')
    form = FileSystemAdminForm


admin.site.register(FileSystem, admin_class=FileSystemAdmin)
admin.site.register(Project, admin_class=ProjectAdmin)



from django.conf.urls import patterns, url

from archer.appsetup import views

urlpatterns = patterns('',
                       url(r'^$', views.index, name='index'),
                       url(r'^admin/$', views.admin, name='admin'),
                       url(r'^projects/$', views.projects, name='projects'),
                       url(r'^filesystems/$', views.filesystems, name='filesystems'),
)


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'FileSystem'
        db.create_table(u'projects_filesystem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('mount_point', self.gf('django.db.models.fields.CharField')(unique=True, max_length=2000)),
        ))
        db.send_create_signal(u'projects', ['FileSystem'])

        # Adding model 'Project'
        db.create_table(u'projects_project', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('file_system', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['projects.FileSystem'])),
            ('directory', self.gf('django.db.models.fields.CharField')(max_length=200)),
        ))
        db.send_create_signal(u'projects', ['Project'])

        # Adding unique constraint on 'Project', fields ['file_system', 'directory']
        db.create_unique(u'projects_project', ['file_system_id', 'directory'])


    def backwards(self, orm):
        # Removing unique constraint on 'Project', fields ['file_system', 'directory']
        db.delete_unique(u'projects_project', ['file_system_id', 'directory'])

        # Deleting model 'FileSystem'
        db.delete_table(u'projects_filesystem')

        # Deleting model 'Project'
        db.delete_table(u'projects_project')


    models = {
        u'projects.filesystem': {
            'Meta': {'object_name': 'FileSystem'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mount_point': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '2000'})
        },
        u'projects.project': {
            'Meta': {'unique_together': "(('file_system', 'directory'),)", 'object_name': 'Project'},
            'directory': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'file_system': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['projects.FileSystem']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['projects']


# noinspection PyUnresolvedReferences
from archer.settings.common import Common


class Dev(Common):
    DEBUG = True
    TEMPLATE_DEBUG = DEBUG

    MIDDLEWARE_CLASSES = Common.MIDDLEWARE_CLASSES + (
        'debug_toolbar.middleware.DebugToolbarMiddleware',
    )

    INSTALLED_APPS = Common.INSTALLED_APPS + (
        'debug_toolbar',
        'gunicorn',
    )

    ## Debug Toolbar

    DEBUG_TOOLBAR_PANELS = (
        'debug_toolbar.panels.version.VersionDebugPanel',
        'debug_toolbar.panels.timer.TimerDebugPanel',
        'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
        'debug_toolbar.panels.headers.HeaderDebugPanel',
        'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
        'debug_toolbar.panels.template.TemplateDebugPanel',
        'debug_toolbar.panels.sql.SQLDebugPanel',
        'debug_toolbar.panels.signals.SignalDebugPanel',
        'debug_toolbar.panels.logger.LoggingPanel',
    )

    INTERNAL_IPS = ('127.0.0.1',)

    DEBUG_TOOLBAR_CONFIG = {
        'INTERCEPT_REDIRECTS': False,
        # 'SHOW_TOOLBAR_CALLBACK': custom_show_toolbar,
        # 'EXTRA_SIGNALS': ['myproject.signals.MySignal'],
        'HIDE_DJANGO_SQL': False,
        'TAG': 'body', # default: body
        'ENABLE_STACKTRACES': True,
    }
    #- Debug Toolbar



from contextlib import closing
import re
import os
import shutil
import tarfile

from django.db import models
from django.conf import settings
from archer.projects.models import Project
from archer.core import exceptions


class Package(models.Model):
    def __get_upload_path(self, filename):
        return os.path.join(settings.MEDIA_ROOT, ".%s" % self.project, filename).replace('/./', '/')

    class Status:
        """Enumeration for Package status"""
        new, uploaded, unpacking, deployed, cancelled, deleted, undeployed, error = range(8)

    class ContentsError(exceptions.ApplicationError):
        pass

    __STATUSES = dict((value, name) for name, value in vars(Status).items() if not name.startswith('__'))

    project = models.ForeignKey(Project)
    file = models.FileField(upload_to=__get_upload_path, max_length=1024)
    status = models.IntegerField(
        choices=__STATUSES.items(),
        blank=False,
        null=False
    )
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        permissions = (
            ('deploy_package', 'Deploy package'),
            ('remove_package', 'Remove package'),
        )

    def status_name(self):
        return Package.__STATUSES[self.status]

    def can_remove(self):
        return self.status in [Package.Status.deployed, Package.Status.undeployed, Package.Status.uploaded]

    def can_deploy(self):
        if self.status in [Package.Status.uploaded, Package.Status.undeployed, Package.Status.cancelled,
                           Package.Status.deployed]:
            return os.path.isfile(self.file.path) and tarfile.is_tarfile(self.file.path)
        return False

    def filepath(self):
        """
            Returns path to the file but hides the path to MEDIA directory.
            """
        return re.sub('^%s' % settings.MEDIA_ROOT, '', self.file.path)

    def filename(self):
        """
            Returns name of the file without full path.
            """
        replace = os.path.join(settings.MEDIA_ROOT, self.project.alias_path()[1::]) + '/'
        return re.sub('^%s' % replace, '', self.file.path)

    def get_file_list(self):
        """
            Returns list of files in the tarfile.
            """
        path = self.file.path
        if not os.path.isfile(path):
            return None
        if not tarfile.is_tarfile(path):
            return None
        tar = tarfile.open(path)
        names = tar.getnames()
        return names

    def deploy(self, subdir='', force=False):
        """
            Unpacks the tar file at the project directory.
            Any previous content is deleted.

            Returns True if deployment succeeded.
            Raises IOError if file does not exist.
            Raises archer.core.exceptions.ValidationError if file is not a tar file.
            """
        import tarfile

        if self.can_deploy or force:
            file_path = self.file.path
            if not os.path.isfile(file_path):
                self.status = Package.Status.error
                self.save()
                raise IOError('package file ' + self.filepath() + ' does not exist')
            if not tarfile.is_tarfile(file_path):
                self.status = Package.Status.error
                self.save()
                raise Package.ContentsError('package file ' + self.filepath() + ' is not tarball file')
            try:
                # self.project.clear_dir(subdir)
                with closing(tarfile.open(file_path)) as tar:
                    self.status = Package.Status.unpacking
                    self.save()

                    dir = self.project.subdir(subdir)
                    for f in tar:
                        if f.name.startswith('/'):
                            raise Package.ContentsError('Tar contains disallowed paths starting with "/"!')
                        normpath = os.path.normpath(f.name)
                        if normpath.startswith('../'):
                            raise Package.ContentsError('Tar contains disallowed paths using ".."!')
                        content_path = os.path.join(dir, f.name)
                        if not os.access(content_path, os.W_OK):
                            shutil.rmtree(content_path, ignore_errors=True)

                    tar.extractall(path=dir)

                    self.status = Package.Status.deployed
                    self.save()
                    return True
            except (Package.ContentsError, tarfile.TarError, IOError, OSError) as e:
                self.status = Package.Status.error
                self.save()
                raise e
        else:
            return False

    def remove(self):
        """
            Removes package file from MEDIA directory.
            """
        result = False
        if os.path.isfile(self.file.path):
            os.unlink(self.file.path)
            result = True
        self.status = Package.Status.deleted
        self.save()
        return result

    def __unicode__(self):
        return self.filename()
